package de.ozgcloud.operator;

import java.time.Duration;
import java.util.Optional;

import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;

class OzgCloudElasticsearchUpdateControlBuilder {

	private OzgCloudElasticsearchCustomResource resource;

	private CustomResourceStatus status;
	private Optional<String> message = Optional.empty();

	private boolean reschedule = false;
	private Duration scheduleDuration;

	public OzgCloudElasticsearchUpdateControlBuilder(OzgCloudElasticsearchCustomResource resource) {
		this.resource = resource;
	}

	public static OzgCloudElasticsearchUpdateControlBuilder fromResource(OzgCloudElasticsearchCustomResource resource) {
		return new OzgCloudElasticsearchUpdateControlBuilder(resource);
	}

	public OzgCloudElasticsearchUpdateControlBuilder withStatus(CustomResourceStatus status) {
		this.status = status;
		return this;
	}

	public OzgCloudElasticsearchUpdateControlBuilder withMessage(String message) {
		this.message = Optional.ofNullable(message);
		return this;
	}

	public OzgCloudElasticsearchUpdateControlBuilder withReschedule(Duration duration) {
		this.reschedule = true;
		this.scheduleDuration = duration;
		return this;
	}

	public UpdateControl<OzgCloudElasticsearchCustomResource> build() {
		resource.setStatus(buildElasticCustomResourceStatus());

		return buildUpdateControl();
	}

	private OzgCloudElasticsearchCustomResourceStatus buildElasticCustomResourceStatus() {
		var userStatus = OzgCloudElasticsearchCustomResourceStatus.builder().status(status);
		message.ifPresent(userStatus::message);

		return userStatus.build();
	}

	private UpdateControl<OzgCloudElasticsearchCustomResource> buildUpdateControl() {
		if (reschedule) {
			return UpdateControl.updateStatus(resource).rescheduleAfter(scheduleDuration);
		}
		return UpdateControl.updateStatus(resource);
	}
}
