package de.ozgcloud.operator.keycloak;

import java.util.Objects;
import java.util.Optional;

import org.keycloak.admin.client.Keycloak;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.RealmRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.springframework.stereotype.Component;

import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@RequiredArgsConstructor
@Component
@Log4j2
public class KeycloakGenericRemoteService {

	private final Keycloak keycloak;

	public Optional<ClientRepresentation> getByClientId(String clientId, String realm) {
		return keycloak.realm(realm).clients().findAll().stream()
				.filter(client -> Objects.equals(client.getClientId(), clientId))
				.findFirst();
	}

	public boolean realmExists(String realmName) {
		return keycloak.realms().findAll().stream()
				.map(RealmRepresentation::getRealm)
				.anyMatch(name -> name.equals(realmName));
	}

	public boolean groupExists(String groupName, String realmName) {
		return keycloak.realm(realmName).groups().groups().stream()
				.anyMatch(group -> Objects.equals(groupName, group.getName()));
	}

	public Optional<RoleRepresentation> getClientRole(String roleName, String realClientId, String realm) {
		LOG.info("Get role {} from client with ID {} in realm {}.", roleName, realClientId, realm);
		return Optional.ofNullable(keycloak.realm(realm).clients().get(realClientId))
				.orElseThrow(() -> new KeycloakException("Client with ID " + realClientId + " for realm " + realm + " not found."))
				.roles()
				.list()
				.stream().filter(role -> Objects.equals(roleName, role.getName()))
				.findFirst();
	}
}
