/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.keycloak.representations.idm.CredentialRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecClientRole;
import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecUserGroup;

@Mapper(unmappedTargetPolicy = ReportingPolicy.IGNORE, unmappedSourcePolicy = ReportingPolicy.IGNORE, collectionMappingStrategy = CollectionMappingStrategy.ACCESSOR_ONLY)
interface KeycloakUserMapper {

	@Mapping(target = "clientRoles", source = "keycloakUser.clientRoles", qualifiedByName = "mapClientRoles")
	@Mapping(target = "credentials", source = ".", qualifiedByName = "mapPassword")
	@Mapping(target = "email", source = "keycloakUser.email")
	@Mapping(target = "emailVerified", constant = "true")
	@Mapping(target = "enabled", constant = "true")
	@Mapping(target = "groups", source = "keycloakUser.groups", qualifiedByName = "mapGroups")
	@Mapping(target = "username", source = "keycloakUser.username")
	@Mapping(target = "firstName", source = "keycloakUser.firstName")
	@Mapping(target = "lastName", source = "keycloakUser.lastName")
	UserRepresentation map(OzgCloudKeycloakUserSpec user);

	@Mapping(target = "clientRoles", source = "keycloakUser.clientRoles", qualifiedByName = "mapClientRoles")
	@Mapping(target = "credentials", source = ".", qualifiedByName = "mapPassword")
	@Mapping(target = "email", source = "keycloakUser.email")
	@Mapping(target = "emailVerified", constant = "true")
	@Mapping(target = "enabled", constant = "true")
	@Mapping(target = "groups", source = "keycloakUser.groups", qualifiedByName = "mapGroups")
	@Mapping(target = "username", source = "keycloakUser.username")
	@Mapping(target = "firstName", source = "keycloakUser.firstName")
	@Mapping(target = "lastName", source = "keycloakUser.lastName")
	UserRepresentation update(@MappingTarget UserRepresentation userRepresentation, OzgCloudKeycloakUserSpec userSpec);

	@Named("mapPassword")
	default List<CredentialRepresentation> mapPassword(OzgCloudKeycloakUserSpec user) {
		return List.of(buildPasswordCredential(user));
	}

	private CredentialRepresentation buildPasswordCredential(OzgCloudKeycloakUserSpec user) {
		var credential = new CredentialRepresentation();
		credential.setType(CredentialRepresentation.PASSWORD);
		credential.setValue(user.getKeycloakUser().getPassword());
		credential.setTemporary(false);

		return credential;
	}

	@Named("mapGroups")
	default List<String> mapGroups(List<KeycloakUserSpecUserGroup> groups) {
		return groups.stream().map(KeycloakUserSpecUserGroup::getName).toList();
	}

	@Named("mapClientRoles")
	default Map<String, List<String>> mapClientRoles(List<OzgCloudKeycloakUserSpec.KeycloakUserSpecClientRole> clientRoles) {
		return clientRoles.stream()
				.collect(Collectors.groupingBy(KeycloakUserSpecClientRole::getClientId,
						Collectors.mapping(KeycloakUserSpecClientRole::getRoleName, Collectors.toList())));
	}
}
