package de.ozgcloud.operator.keycloak.client;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.mockito.Spy;

class ProtocolMapperRepresentationHelperTest {

	@Spy
	private final ProtocolMapperRepresentationHelper helper = new ProtocolMapperRepresentationHelper();

	@DisplayName("Create organisationsEinheitId mapper")
	@Nested
	class TestCreateOrganisationsEinheitIdMapper {

		@Test
		void shouldSetName() {
			var mapper = create();

			assertThat(mapper.getName()).isEqualTo(ProtocolMapperRepresentationHelper.ORGANISATIONS_EINHEIT_ID_MAPPER_NAME);
		}

		@Test
		void shouldSetProtocol() {
			var mapper = create();

			assertThat(mapper.getProtocol()).isEqualTo(ProtocolMapperRepresentationHelper.OPENID_CONNECT_PROTOCOL);
		}

		@Test
		void shouldSetProtocolMapper() {
			var mapper = create();

			assertThat(mapper.getProtocolMapper()).isEqualTo(ProtocolMapperRepresentationHelper.USER_ATTRIBUTE_PROTOCOL_MAPPER);
		}

		@Test
		void shouldBuildConfig() {
			create();

			verify(helper).buildOrganisationsEinheitIdMapperConfig();
		}

		@DisplayName("build config")
		@Nested
		class TestBuildOrganisationsEinheitIdConfig {

			@Test
			void shouldSetAccessTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetAggregateAttrs() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_AGGREGATE_ATTRS_KEY, "true");
			}

			@Test
			void shouldSetClaimName() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
			}

			@Test
			void shouldSetIdTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ID_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetJsonTypeLabel() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_JSON_TYPE_LABEL_KEY, "String");
			}

			@Test
			void shouldSetMultivalued() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_MULTIVALUED_KEY, "true");
			}

			@Test
			void shouldSetUserAttribute() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_ATTRIBUTE_KEY, "organisationseinheitId");
			}

			@Test
			void shouldSetUserInfoTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
			}

			private Map<String, String> buildConfig() {
				return helper.buildOrganisationsEinheitIdMapperConfig();
			}
		}

		private ProtocolMapperRepresentation create() {
			return helper.createOrganisationsEinheitIdMapper();
		}
	}

	@DisplayName("Create ozgCloudUserId mapper")
	@Nested
	class TestCreateOzgCloudUserIdMapper {

		@Test
		void shouldSetName() {
			var mapper = create();

			assertThat(mapper.getName()).isEqualTo(ProtocolMapperRepresentationHelper.OZGCLOUD_USER_ID_MAPPER_NAME);
		}

		@Test
		void shouldSetProtocol() {
			var mapper = create();

			assertThat(mapper.getProtocol()).isEqualTo(ProtocolMapperRepresentationHelper.OPENID_CONNECT_PROTOCOL);
		}

		@Test
		void shouldSetProtocolMapper() {
			var mapper = create();

			assertThat(mapper.getProtocolMapper()).isEqualTo(ProtocolMapperRepresentationHelper.USER_ATTRIBUTE_PROTOCOL_MAPPER);
		}

		@Test
		void shouldBuildConfig() {
			create();

			verify(helper).buildOzgCloudUserIdMapperConfig();
		}

		@DisplayName("build config")
		@Nested
		class TestBuildOrganisationsEinheitIdConfig {

			@Test
			void shouldSetAccessTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetAggregateAttrs() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_AGGREGATE_ATTRS_KEY, "false");
			}

			@Test
			void shouldSetClaimName() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_CLAIM_NAME_KEY, "ozgCloudUserId");
			}

			@Test
			void shouldSetIdTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ID_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetJsonTypeLabel() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_JSON_TYPE_LABEL_KEY, StringUtils.EMPTY);
			}

			@Test
			void shouldSetMultivalued() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_MULTIVALUED_KEY, "false");
			}

			@Test
			void shouldSetUserAttribute() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_ATTRIBUTE_KEY, "ozgCloudUserId");
			}

			@Test
			void shouldSetUserInfoTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
			}

			private Map<String, String> buildConfig() {
				return helper.buildOzgCloudUserIdMapperConfig();
			}
		}

		private ProtocolMapperRepresentation create() {
			return helper.createOzgCloudUserIdMapper();
		}
	}

	@DisplayName("Create organisationsEinheitIdLdap mapper")
	@Nested
	class TestCreateOrganisationsEinheitIdLdapMapper {

		@Test
		void shouldSetName() {
			var mapper = create();

			assertThat(mapper.getName()).isEqualTo(ProtocolMapperRepresentationHelper.ORGANISATIONS_EINHEIT_ID_LDAP_MAPPER_NAME);
		}

		@Test
		void shouldSetProtocol() {
			var mapper = create();

			assertThat(mapper.getProtocol()).isEqualTo(ProtocolMapperRepresentationHelper.OPENID_CONNECT_PROTOCOL);
		}

		@Test
		void shouldSetProtocolMapper() {
			var mapper = create();

			assertThat(mapper.getProtocolMapper()).isEqualTo(ProtocolMapperRepresentationHelper.USER_ATTRIBUTE_PROTOCOL_MAPPER);
		}

		@Test
		void shouldBuildConfig() {
			create();

			verify(helper).buildOrganisationsEinheitIdLdapMapperConfig();
		}

		@DisplayName("build config")
		@Nested
		class TestBuildOrganisationsEinheitIdConfig {

			@Test
			void shouldSetAccessTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetAggregateAttrs() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_AGGREGATE_ATTRS_KEY, "true");
			}

			@Test
			void shouldSetClaimName() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
			}

			@Test
			void shouldSetIdTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ID_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetJsonTypeLabel() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_JSON_TYPE_LABEL_KEY, "int");
			}

			@Test
			void shouldSetMultivalued() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_MULTIVALUED_KEY, "true");
			}

			@Test
			void shouldSetUserAttribute() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_ATTRIBUTE_KEY, "extensionAttribute1");
			}

			@Test
			void shouldSetUserInfoTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
			}

			private Map<String, String> buildConfig() {
				return helper.buildOrganisationsEinheitIdLdapMapperConfig();
			}
		}

		private ProtocolMapperRepresentation create() {
			return helper.createOrganisationeEinheitIdLdapMapper();
		}
	}

	@DisplayName("Create clientRoles mapper")
	@Nested
	class TestCreateClientRolesMapper {

		@Test
		void shouldSetName() {
			var mapper = create();

			assertThat(mapper.getName()).isEqualTo(ProtocolMapperRepresentationHelper.CLIENT_ROLES_MAPPER_NAME);
		}

		@Test
		void shouldSetProtocol() {
			var mapper = create();

			assertThat(mapper.getProtocol()).isEqualTo(ProtocolMapperRepresentationHelper.OPENID_CONNECT_PROTOCOL);
		}

		@Test
		void shouldSetProtocolMapper() {
			var mapper = create();

			assertThat(mapper.getProtocolMapper()).isEqualTo(ProtocolMapperRepresentationHelper.USER_CLIENT_ROLE_PROTOCOL_MAPPER);
		}

		@Test
		void shouldBuildConfig() {
			create();

			verify(helper).buildClientRolesMapperConfig();
		}

		@DisplayName("build config")
		@Nested
		class TestBuildOrganisationsEinheitIdConfig {

			@Test
			void shouldSetAccessTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetClaimName() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_CLAIM_NAME_KEY, "resource_access.${client_id}.roles");
			}

			@Test
			void shouldSetIdTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ID_TOKEN_CLAIM_KEY, "false");
			}

			@Test
			void shouldSetJsonTypeLabel() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_JSON_TYPE_LABEL_KEY, StringUtils.EMPTY);
			}

			@Test
			void shouldSetMultivalued() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_MULTIVALUED_KEY, "true");
			}

			@Test
			void shouldSetUserModelClientRoleMappingClientId() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_CLIENT_ID,
						StringUtils.EMPTY);
			}

			@Test
			void shouldSetUserModelClientRoleMappingRolePrefix() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_ROLE_PREFIX,
						StringUtils.EMPTY);
			}

			private Map<String, String> buildConfig() {
				return helper.buildClientRolesMapperConfig();
			}
		}

		private ProtocolMapperRepresentation create() {
			return helper.createClientRolesMapper();
		}
	}
}
