package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Spy;

class UserNameConverterTest {

	@Spy
	private UserNameConverter converter = new UserNameConverter();

	@DisplayName("To secretName")
	@Nested
	class TestToSecretName {

		@Test
		void shouldReturnName() {
			var secretName = converter.toSecretName(KeycloakUserSpecUserTestFactory.create());

			assertThat(secretName).isEqualTo(KeycloakUserSpecUserTestFactory.USERNAME.toLowerCase() + "-credentials");
		}

		@Test
		void shouldClarifyUserName() {
			converter.toSecretName(KeycloakUserSpecUserTestFactory.create());

			verify(converter).clarifyName(KeycloakUserSpecUserTestFactory.USERNAME.toLowerCase());
		}

		@DisplayName("clarify name")
		@Nested
		class TestClarifyName {

			@ValueSource(strings = { "_user_name_", ".user.name.", "-user-name-" })
			@ParameterizedTest
			void shouldReplaceForbiddenCharacter(String userName) {
				var clarifiedName = converter.clarifyName(userName);

				assertThat(clarifiedName).isEqualTo("username");
			}
		}
	}
}